package at.jku.risc.stout.urauc;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Set;

import at.jku.risc.stout.urauc.algo.AlignFnc;
import at.jku.risc.stout.urauc.algo.AlignFncInput;
import at.jku.risc.stout.urauc.algo.AntiUnify;
import at.jku.risc.stout.urauc.algo.AntiUnifyProblem;
import at.jku.risc.stout.urauc.algo.AntiUnifySystem;
import at.jku.risc.stout.urauc.algo.DebugLevel;
import at.jku.risc.stout.urauc.algo.IllegalAlignmentException;
import at.jku.risc.stout.urauc.algo.JustificationException;
import at.jku.risc.stout.urauc.data.Alignment;
import at.jku.risc.stout.urauc.data.EquationSystem;
import at.jku.risc.stout.urauc.data.Hedge;
import at.jku.risc.stout.urauc.data.InputParser;
import at.jku.risc.stout.urauc.data.MalformedTermException;
import at.jku.risc.stout.urauc.data.NodeFactory;
import at.jku.risc.stout.urauc.data.atom.TermAtom;
import at.jku.risc.stout.urauc.data.atom.Variable;
import at.jku.risc.stout.urauc.util.ControlledException;
import at.jku.risc.stout.urauc.util.DataStructureFactory;

public class AntiUnifyConsole {
	private DebugLevel debugLevel;
	private PrintStream info = System.out;
	private PrintStream debug = System.out;

	public AntiUnifyConsole(DebugLevel debugLevel) {
		this.debugLevel = debugLevel;
	}

	public AntiUnifyConsole(DebugLevel debugLevel, PrintStream info,
			PrintStream debug) {
		this(debugLevel);
		this.info = info;
		this.debug = debug;
	}

	public void exec(String aupsIn, String alignIn, boolean iterateAll,
			boolean justify, boolean applyResH, final boolean equalityFilter,
			boolean commRoot, Set<TermAtom> commFnc, Set<TermAtom> assocFnc,
			AlignFnc aFnc, int minLen, int upperBound)
			throws MalformedTermException, IllegalAlignmentException,
			ControlledException, JustificationException {

		NodeFactory.PREFIX_FreshCtxVar = getExists(aupsIn, "X", "Y", "Z", "U",
				"V", "W", "$");
		NodeFactory.PREFIX_FreshHedgeVar = getExists(aupsIn, "x", "y", "z",
				"u", "v", "w", "#");
		NodeFactory.resetCounter(); // reset fresh variable counter

		EquationSystem<AntiUnifyProblem> sys = new EquationSystem<AntiUnifyProblem>() {
			@Override
			public AntiUnifyProblem newEquation() {
				return new AntiUnifyProblem();
			}
		};
		new InputParser<AntiUnifyProblem>(sys).parseEquationSystem(aupsIn);
		for (AntiUnifyProblem aup : sys) {
			info.print(aup.getLeft());
			info.print(AntiUnifyProblem.PRINT_EQ_SEPARATOR);
			info.println(aup.getRight());
		}
		info.println();

		if (aFnc instanceof AlignFncInput) {
			if (sys.size() != 1)
				throw new IllegalAlignmentException(
						"Alignment input is only supported for one equation!");
			Alignment al = InputParser.parseAlignment(sys.getFirst(), alignIn,
					true);
			((AlignFncInput) aFnc).setAlignment(al);
		}

		for (AntiUnifyProblem aup : sys) {
			aup.makeCommutativ(commFnc, commRoot);
			aup.makeAssociative(assocFnc);
		}

		final Set<Hedge> filter = DataStructureFactory.$.newSet();
		new AntiUnify(aFnc, sys, minLen, upperBound, applyResH, debugLevel) {
			public void callback(AntiUnifySystem res, Variable var) {
				if (equalityFilter || debugLevel == DebugLevel.SILENT) {
					Hedge result = res.getSigma().get(var);
					result.normalizeVarNames(new HashMap<String, String>());
					filter.add(result);
				}
			};
		}.antiUnify(iterateAll, justify, debug);

		if (equalityFilter) {
			if (debugLevel != DebugLevel.SILENT) {
				debug.println();
				debug.println("FILTERED RESULTS: (commutative equality under variable renaming)");
			}
			for (Hedge result : filter)
				debug.println(result);
			if (debugLevel != DebugLevel.SILENT) {
				debug.println();
				debug.println(filter.size() + " different results found.");
			}
		} else if (debugLevel == DebugLevel.SILENT) {
			debug.println(filter.size() + " results found.");
		}
	}

	private static String getExists(String in, String... choice)
			throws ControlledException {
		for (String c : choice)
			if (in.indexOf(c) == -1)
				return c;
		throw new IllegalArgumentException("No choice was possible");
	}

}
