/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urauc;

import java.io.PrintStream;
import java.util.HashSet;
import java.util.Set;

import at.jku.risc.stout.urauc.algo.AlignFnc;
import at.jku.risc.stout.urauc.algo.AlignFncLAA;
import at.jku.risc.stout.urauc.algo.AntiUnify;
import at.jku.risc.stout.urauc.algo.AntiUnifyProblem;
import at.jku.risc.stout.urauc.algo.AntiUnifySystem;
import at.jku.risc.stout.urauc.algo.DebugLevel;
import at.jku.risc.stout.urauc.data.EquationSystem;
import at.jku.risc.stout.urauc.data.InputParser;
import at.jku.risc.stout.urauc.data.atom.Variable;
import at.jku.risc.stout.urauc.util.ControlledException;

public class TestAntiUnify {
	public static void main(String[] args) {
		try {
			// String l = "f(a),f(a)";
			// String r = "f(a),f";
			// String l = "f(a,b,c),g(a),h(a)";
			// String r = "f(a,b,c),g(a),h(a)";
			// String l = "f(g(a,X),a,X,b)";
			// String r = "f(g(b),b)";
			// String l = "f(g(a,a),a,X,b)";
			// String r = "f(g(b,b),g(Y),b)";
			// String l =
			// "if(geq(x1, x2), then(eq(x3, add(x4, x2)), eq(x4, add(x4, 1))), else(eq(x3, sub(x4, x1)))) ";
			// String r =
			// "if(geq(y1, y2), then(eq(y3, add(y4, y2)), eq(y5, 1), eq(y4, add(y4, 5))), else(eq(y3, sub(y4, y1))))";
			String in = "sumProd(input(type(int),n), returnType(_void),eq(type(float),sum,0),eq(type(float),prod,1),for(eq(type(int),i,1),leq(i,n),inc(i),eq(sum,add(sum,i)),eq(prod,mult(prod,i)),foo(sum,prod)))"
					+ " =^= "
					+ "sumProd(input(type(int),n), returnType(_void), eq(type(float),sum,0),eq(type(float),prod,1),for(eq(type(int),i,1),leq(i,n),inc(i),eq(sum,add(sum,i)),foo(sum,prod)))";
			in = "sumProd(input(type(int),n), returnType(_void),eq(type(float),sum,0),eq(type(float),prod,1),for(eq(type(int),i,1),leq(i,n),inc(i),eq(sum,add(sum,i)),eq(prod,mult(prod,i)),foo(sum,prod)))"
					+ " =^= "
					+ "sumProd(input(type(int),n), returnType(_void),eq(type(float),sum,0),eq(type(float),prod,1),eq(type(int),i,1),while(leq(i,n),eq(sum,add(sum,i)),eq(prod,mult(prod,i)),foo(sum,prod),inc(i)))";
			// String in = "f(x),f(f(g(a,a)), a, a) =^= x,f(g(b,b), b, b)";
			// String in = "f(h(c,a,c,a), a, a) =^= f(g(c,b,c,a), b, b)";
			// String in = "f(a,b,a), d =^= g(a,c,a,d)";
			// String in =
			// "x,f(f(g(a,a)), a, f(a), a) =^= x,f(g(a,b), b, a, b)";
			// String in = "f(b,a), d =^= g(c,a,d)";
			in =
			 "a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a =^= a(a),a(a),a(a),a(a),a(a),a(a),a(a),a(a),a(a)";
			// String in = "f(a, f(b, b)), c =^= f(a, b), b, g(c)";
			// String in =
			// "f(a, g(h(a, h(b, b, b)), d)) =^= f(a, a, h(g(c, b)))";
//			in = "h(a), f(h(g(a, b, b),c), b, b) =^= a, f(g(a,d), c, d); a,a =^= a";
//			in = "if(eq(n,0), 1,    if(eq(n,1), times(1,1),           if(eq(n,2), times(2,times(1,1)),                  if(eq(n,3), times(3,times(2,times(1,1))))           )    )) =^= if(eq(n,0), 0,    if(eq(n,1), subtract(0,1),           if(eq(n,2), subtract(subtract(0,1),2),                  if(eq(n,3), subtract(subtract(subtract(0,1),2),3))           )    ))";
//			in = "if(eq(n,0), 1,    if(eq(n,1), times(_,1,1,_),           if(eq(n,2), times(times(_,1,1,_),2),                  if(eq(n,3), times(times(times(_,1,1,_),2),3))           )    )) =^= if(eq(n,0), 0,    if(eq(n,1), subtract(_,0,1,_),           if(eq(n,2), subtract(subtract(_,0,1,_),2),                  if(eq(n,3), subtract(subtract(subtract(_,0,1,_),2),3))           )    ))";
//			in = "if(eq(n,0), 1,    if(eq(n,1), times(_,1,1,_),           if(eq(n,2), times(_,times(_,1,1,_),2,_),                  if(eq(n,3), times(_,times(_,times(_,1,1,_),2,_),3,_))           )    )) =^= if(eq(n,0), 0,    if(eq(n,1), subtract(_,0,1,_),           if(eq(n,2), subtract(_,subtract(_,0,1,_),2,_),                  if(eq(n,3), subtract(_,subtract(_,subtract(_,0,1,_),2,_),3,_))           )    ))";
			AlignFnc aFnc = new AlignFncLAA();
			// TermAtom.DEBUG_POSITION = true;
			new TestAntiUnify().test(in, aFnc, true, DebugLevel.SIMPLE);
			// new TestAntiUnify().runTests();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void runTests() throws ControlledException {
		runOne("a,a,a,a,a,a =^= a(a),a(a),a(a)");
		runOne("a =^= b");
		runOne("f(a),f(a) =^= f(a),f");
		runOne("f(a,b,c),g(a),h(a) =^= f(a,b,c),g(a),h(a)");
		runOne("f(g(a,X),a,X,b) =^= f(g(b),b)");
		runOne("f(g(a,a),a,X,b) =^= f(g(b,b),g(Y),b)");
		runOne("if(geq(x1, x2), then(eq(x3, add(x4, x2)), eq(x4, add(x4, 1))), else(eq(x3, sub(x4, x1)))) =^= if(geq(y1, y2), then(eq(y3, add(y4, y2)), eq(y5, 1), eq(y4, add(y4, 5))), else(eq(y3, sub(y4, y1))))");
		runOne("sumProd(input(type(int), n), returnType(bvoid), eq(type(float), n, 0.0), eq(type(float), prod, 1.0), for(eq(type(int), i, 1), le(i, n), pp(i), eq(sum, plus(sum, i)), eq(prod, mult(prod, i)), foo(sum, prod))) =^= sumProd(input(type(int), n), returnType(bvoid), eq(type(float), n, 0.0), eq(type(float), prod, 1.0), for(eq(type(int), i, 1), le(i, n), pp(i), eq(sum, plus(sum, mult(i, i))), eq(prod, mult(prod, mult(i, i))), foo(sum, prod)))");
		runOne("f(x),f(f(g(a,a)), a, a) =^= x,f(g(b,b), b, b)");
		runOne("f(x),f(f(h(a,a)), a, a) =^= x,f(g(b,b), b, b)");
		runOne("f(h(c,a,c,a), a, a) =^= f(g(c,b,c,a), b, b)");
		runOne("a, a, b, b, f, f, f(a, a, b, b) =^= a, a, c, f, f, f(a, a, c)");
		runOne("f(a,b,a), d =^= g(a,c,a,d)");
		runOne("x,f(f(g(a,a)), a, f(a), a) =^= x,f(g(a,b), b, a, b)");
		runOne("f(b,a), d =^= g(c,a,d)");
		runOne("f(a, f(b, b)), c =^= f(a, b), b, g(c)");
		runOne("f(a, g(h(a, h(b, b, b)), d)) =^= f(a, a, h(g(c, b)))");
		runOne("f(g(c,c,a)), a =^= f(c,c,b), b");
		runOne("f(g(c,a)), a =^= f(c,b), b");
		runOne("f(f(a),a)=^=g(a,a)");
		runOne("f(f(a),a)=^=g(g(a),a)");
	}

	public void runOne(String in) throws ControlledException {
		new TestAntiUnify()
				.test(in, new AlignFncLAA(), true, DebugLevel.SIMPLE);
	}

	private void test(String in, AlignFnc aFnc, boolean justify,
			DebugLevel debugLevel) throws ControlledException {
		EquationSystem<AntiUnifyProblem> sys = new EquationSystem<AntiUnifyProblem>() {
			@Override
			public AntiUnifyProblem newEquation() {
				return new AntiUnifyProblem();
			}
		};
		new InputParser<AntiUnifyProblem>(sys).parseEquationSystem(in);
		PrintStream out = System.out;
		out.println();
		for (AntiUnifyProblem aup : sys) {
			out.print(aup.getLeft());
			out.print(AntiUnifyProblem.PRINT_EQ_SEPARATOR);
			out.println(aup.getRight());
		}
		out.println();

		final Set<String> results = new HashSet<String>();
		new AntiUnify(aFnc, sys, DebugLevel.VERBOSE) {
			@Override
			public void callback(AntiUnifySystem result,
					Variable generalizationVar) {
				results.add(String.valueOf(result.getSigma().get(
						generalizationVar)));
			}
		}.antiUnify(true, justify, out);
		for (String string : results) {
			out.println(string);
		}
	}
}
