/*
 * Copyright 2014 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urauc;

import java.util.Set;

import at.jku.risc.stout.urauc.algo.AntiUnifyProblem;
import at.jku.risc.stout.urauc.algo.AntiUnifyProblem.CommutativeArrangementIter;
import at.jku.risc.stout.urauc.data.EquationSystem;
import at.jku.risc.stout.urauc.data.InputParser;
import at.jku.risc.stout.urauc.data.atom.FunctionSymbol;
import at.jku.risc.stout.urauc.data.atom.TermAtom;
import at.jku.risc.stout.urauc.util.DataStructureFactory;

public class TestCommutative {

	static Set<TermAtom> assoc = DataStructureFactory.$.newSet();

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		assoc.add(new FunctionSymbol("f"));
		testCommutative(
				"f(c,d,x,g(Y,x,b),h(y,X,X(f(),g),F(f,g))) =^= x,g(f(b,c,d,f(b,a,a)),g(a,b)),g,f",
				"g");
		testCommutative(
				"f(c,d,x,g(Y,x,b),h(y,X,X(f(),g),F(f,g))) =^= x,g(f(b,c,d,f(b,a,a)),g(a,b)),g,f",
				"f");
		testCommutative(
				"f(c,d,x,g(Y,x,b),h(y,X,X(f(),g),F(f,g))) =^= x,g(f(b,c,d,f(b,a,a)),g(a,b)),g,f",
				"g", "f");
		testCommutative(
				"f(c,d,x,g(Y,x,b),h(y,X,X(f(),g),F(f,g))) =^= x,g(f(b,c,d,f(b,a,a)),g(a,b)),g,f",
				"g", "f", "h", "X", "F");
	}

	public static void testCommutative(String in, String... commutative) {
		try {
			EquationSystem<AntiUnifyProblem> sys = new EquationSystem<AntiUnifyProblem>() {
				@Override
				public AntiUnifyProblem newEquation() {
					return new AntiUnifyProblem();
				}
			};
			new InputParser<AntiUnifyProblem>(sys).parseEquationSystem(in);
			Set<TermAtom> comm = DataStructureFactory.$.newSet();
			for (String symb : commutative)
				comm.add(new FunctionSymbol(symb));

			for (AntiUnifyProblem aup : sys) {
				aup.makeCommutativ(comm, false);
				aup.makeAssociative(assoc);
			}

			final INT i = new INT();
			// TermAtom.DEBUG_POSITION=true;
			for (AntiUnifyProblem aup : sys) {
				for (CommutativeArrangementIter iter = aup
						.iteratorHorizontalPartX(true); iter.hasNext();) {
					iter.next();
					i.i++;
					// System.out.println(getAUP());
				}
			}
			System.out.println("# arrangements: " + i.i);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static class INT {
		public int i;
	}
}
