/*
 * Copyright 2014 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package at.jku.risc.stout.urauc;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import at.jku.risc.stout.urauc.algo.AlignFncLAA;
import at.jku.risc.stout.urauc.algo.AntiUnify;
import at.jku.risc.stout.urauc.algo.AntiUnifyProblem;
import at.jku.risc.stout.urauc.algo.AntiUnifySystem;
import at.jku.risc.stout.urauc.algo.DebugLevel;
import at.jku.risc.stout.urauc.algo.JustificationException;
import at.jku.risc.stout.urauc.data.EquationSystem;
import at.jku.risc.stout.urauc.data.Hedge;
import at.jku.risc.stout.urauc.data.InputParser;
import at.jku.risc.stout.urauc.data.atom.Variable;
import at.jku.risc.stout.urauc.util.ControlledException;

public class TestConsecutive {

	public static void main(String[] args) throws IOException,
			JustificationException, ControlledException {

		// The first term
		Reader t0 = new StringReader("f(b,a,a)"); // Use FileReader instead

		// List of generalizations can be initialized by the first term.
		final List<Hedge> generalizations = new ArrayList<>();
		Hedge h = new Hedge();
		h.add(new InputParser<>(null).parseTerm(t0));
		generalizations.add(h);

		// The other terms
		StringReader[] t1_tn = new StringReader[] {
				new StringReader("f(a,a,b)"), new StringReader("f(a,c,a,c)") };

		for (StringReader term : t1_tn) {
			Hedge hNext = new Hedge();
			hNext.add(new InputParser<>(null).parseTerm(term));

			EquationSystem<AntiUnifyProblem> eqSys = new EquationSystem<AntiUnifyProblem>() {
				public AntiUnifyProblem newEquation() {
					return new AntiUnifyProblem();
				}
			};

			for (Hedge genOld : generalizations) {
				AntiUnifyProblem aup = new AntiUnifyProblem(genOld, hNext, null);
				aup.init();
				eqSys.add(aup);
			}

			generalizations.clear();
			new AntiUnify(new AlignFncLAA(), eqSys, DebugLevel.SILENT) {
				public void callback(AntiUnifySystem res, Variable var) {
					Hedge h = res.getSigma().get(var);
					generalizations.add(h);
				};
			}.antiUnify(true, false, null);
		}

		System.out.println(generalizations);
	}

}
