/*
 * Copyright 2014 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urauc.util;

import java.util.Arrays;

/**
 * A flexible sized list of primitive integers.
 * 
 * @author Alexander Baumgartner
 */
public class IntList implements Cloneable {
	public int size;
	public long[] values;

	public IntList() {
		this(new long[16]);
	}

	public IntList(int initCapacity) {
		this(new long[initCapacity]);
	}

	public IntList(long[] packingArray) {
		this.values = packingArray;
	}

	public void add(long val) {
		ensureCapacity(size);
		values[size] = val;
		size++;
	}

	public void addAll(IntList other) {
		ensureCapacity(size + other.size);
		System.arraycopy(other.values, 0, values, size, other.size);
		size = size + other.size;
	}

	private void ensureCapacity(int newSize) {
		if (newSize >= values.length)
			values = Arrays.copyOf(values, newSize + (size >> 1));
	}

	@Override
	public IntList clone() {
		return new IntList(Arrays.copyOf(values, size));
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append('[');
		if (size > 0) {
			sb.append(values[0]);
			for (int i = 0; ++i < size;)
				sb.append(',').append(values[i]);
		}
		sb.append(']');
		return sb.toString();
	}
}